/*******************************************************************************
File name:	tcps1.c
 
Description:
	This program demo 
	how to use socket lib coding TCP Server program on UC7000/DA-600 series.
	
	The Hardware connection is following:
	[UC7000/DA-600 series]--- ~~LAN~~ ---PC Host Example (Windows platform)
	
	If data received from Ethernet,send them all back to the LAN port.
	1. Create TCP/IP socket. 
	2. Socket Listen on TCP Port 4001.
	3. If TCP connection is established, R/W data.
	
Usage:
    	1. Make downloading file (tcps1-release or tcps1-debug) for UC7000/DA-600 series 
    	    1.1 Create executable file from source code(*.c) 
    	    	1.1.1 Modify tcps1.c: 
    	    		change "TCPPORT" if needed.
    	    	1.1.2 Compiler and link to make tcps1-release (no debug message) or tcps1-debug (with debug message) 
	2. Download executable file to UC7000/DA-600 series 
    	3. Run Application on UC7000/DA-600 series 
    	4. Execute TCP Client example for PC.[tcpclient.exe or frmClient.exe]
    	   input the TCP Port number equal to "TCPPORT"
    	         UC7000/DA-600 series Server ip equal to UC7000/DA-600 series IP adddress.
    	5. user can check if the sended data received back. 

History:    
    	Version		Author		Date		Comment
        1.0		Victor Yu.	01-15-2004      Wrote it.
*******************************************************************************/
#include        <stdio.h>
#include        <stdlib.h>
#include        <sys/types.h>
#include        <sys/socket.h>
#include        <netinet/in.h>
#include        <errno.h>

#define TCPPORT	4001

#define TESTLEN	1024

char buf[TESTLEN]; 

int main()
{
	int			sListen, sClient;
	int 			size,len;	
	struct sockaddr_in	local, client;
	
	printf("create socket......");
	if ( (sListen=socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 ){
		printf("fail [%d]\n", errno);
		return 0;
	} else 
		printf("OK\n");

	local.sin_addr.s_addr = htonl(INADDR_ANY);
	local.sin_family = AF_INET;
	local.sin_port = htons(TCPPORT);

	printf("socket bind......");
	if ( bind(sListen, (struct sockaddr *)&local, sizeof(local)) < 0 ) {
		printf("fail [%d]\n", errno);
		return 0;
	} else
		printf("OK\n");

	printf("socket listen......");
	if (listen(sListen,1) < 0) {
		printf("fail [%d]\n", errno);
		return 0;
	} else
		printf("OK\n");

	size = sizeof(client);
        printf("accept......");
	if ( (sClient=accept(sListen, (struct sockaddr *)&client, &size)) < 0 ){
		printf("fail [%d]", errno);
		return 0;
	} else 
		printf("OK\n");

        printf("ready to recv & write data\n");
	while(1) {
		len = recv(sClient, buf, TESTLEN, 0);
		if ( len > 0 ) {	
			if (send(sClient,buf,len,0) <= 0) {
				printf("send fail [%d]\n", errno);
				break;
                	}
            	} else if ( len == 0 ){
			printf("disconnect\n");
	    		break;
	    	} else if ( len < 0 ) {
			printf("recv fail [%d]", errno);
			break;
            	} 
	}

	printf("close socket\n");	
	close(sListen);
	close(sClient);

	return 0;
}
