/*******************************************************************************
File Name : logger.c

Description : 
	This program demo how to log serial data.
	
Usage : 
	1.Compile this file and execute on target platform.
	2.The program will log to a log file describe in config.txt

History : 
	Version		Author		Date		Comment
	1.0		AceLan		01-05-2006	Using library
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <time.h>

#include "serial.h"
#include "keypad.h"
#include "lcm.h"

int port1, port2, mode;
int logfd1= -1, logfd2= -1;

int fp, portfd1, portfd2;

char tmp[255], logname1[ 255], logname2[ 255], buf[ 1024];
int baud_rate, parity, data_bits, stop_bit, flow;
int i, j, k, len= 0;

fd_set readfds;
int tmpfd, maxfd= -1;

time_t	tt;

void my_exit()
{
	printf("Stop loggin..\n");
	printf("Close ports\n");
	SerialClose( port1);
	SerialClose( port2);
	
	printf("Close log files\n");
	close( logfd1);
	close( logfd2);

	system("/bin/showiponlcm");

	exit( 0);
}

void RotateFile( char* file)
{
	char tmp[ 80], file1[ 255], file2[ 255], cmd[ 255];
	char *head, *ext;
	int i, dot;

	for( i= 0; i< strlen( file); i++)
		if( file[ i] == '.')
			break;
	dot= i;
	head= file;
	file[ dot]= '\0';
	ext= file+ dot+ 1;
	
	for( i= 10; i > 1; i--)
	{
		sprintf( file1, "%s-%d.%s", head, i, ext);
		if( i- 1 == 1)
			sprintf( file2, "%s.%s", file, ext);
		else
			sprintf( file2, "%s-%d.%s", file, i- 1, ext);
		sprintf( cmd, "mv %s %s", file2, file1);
		system( cmd);
	}
	file[ dot]= '.';
}

void StartLogger()
{
	if( logfd1 == -1)
	{
		RotateFile( logname1);
		RotateFile( logname2);

		logfd1=open( logname1, O_RDWR | O_CREAT);
		logfd2=open( logname2, O_RDWR | O_CREAT);
	}
}

void StopLogger()
{
	logfd1= -1;
	logfd2= -1;
}

int main( int argc, char* argv[])
{
	int key;

	struct timeval tv;
	tv.tv_sec = 1;
	tv.tv_usec = 0;

	signal( SIGINT, my_exit);
	signal( SIGTERM, my_exit);

	if( argc != 2)
	{
		printf("Usage: %s config_file\n", argv[ 0]);
		exit( -1);
	}

	fp=open( argv[ 1], O_RDWR);
	len= read( fp, buf, 1024);

	i= j= k= 0;
	while( i < len)
	{
		while( buf[ j] != 10)
			j++;
		
		if( k == 0)
			strncpy( logname1, buf+ i, j- i);
		else if( k == 1)
			strncpy( logname2, buf+ i, j- i);
		else if( k == 2)
			port1= buf[ j- 1]- '0'- 1;
		else if( k == 3)
			port2= buf[ j- 1]- '0'- 1;
		else if( k == 4)
			mode= buf[ j- 1]- '0';
		else if( k == 5)
		{
			strncpy( tmp, buf+ i, j- i);
			tmp[ j- i]= '\0';
			baud_rate= atoi( tmp);
		}
		else if( k == 6)
			parity= buf[ j- 1]- '0';
		else if( k == 7)
			data_bits= buf[ j- 1]- '0';
		else if( k == 8)
			stop_bit= buf[ j- 1]- '0';
		else if( k == 9)
			flow= buf[ j- 1]- '0';
		
		i= j+ 1;
		j++, k++;
	}

	SerialOpen( port1);
	SerialOpen( port2);

	portfd1= FindFD( port1);
	portfd2= FindFD( port2);
	
	if( SerialSetMode( port1, mode) < 0)
	{
		printf("SerialSetMode error!!\n");
		exit( -1);
	}
	if( SerialSetMode( port2, mode) < 0)
	{
		printf("SerialSetMode error!!\n");
		exit( -1);
	}
	if( SerialGetMode( port1) != mode)
	{
		printf("Mode error!!\n");
		exit( -1);
	}
	
	SerialSetSpeed( port1, baud_rate);
	SerialSetSpeed( port2, baud_rate);

	SerialSetParam( port1, parity, data_bits, stop_bit);
	SerialSetParam( port2, parity, data_bits, stop_bit);

	SerialFlowControl( port1, flow);
	SerialFlowControl( port2, flow);

	maxfd= portfd1;
	if( portfd2 > maxfd)
		maxfd= portfd2;
		
	printf("Sniff Port%d and Port%d\n", port1+ 1, port2+ 1);
	printf("Log file: %s and %s\n", logname1, logname2);
	printf("Start to log..\n");
	while( 1)
	{
		key= KeypadGetKey();

		if( key != -1)
			printf("F%d pressed\n", key+ 1);
		switch( key)
		{
		case 0:			// F1
			printf("Start logging..\n");
			StartLogger();
			LCMClear();
			break;
		case 1:			// F2
			printf("Stop logging..\n");
			StopLogger();
			break;
		case 2:			// F3
			system("/bin/showiponlcm");
			break;
		case 3:			// F4
			break;
		case 4:			// F5
			break;
		}

		FD_ZERO( &readfds);
		FD_SET( portfd1, &readfds);
		FD_SET( portfd2, &readfds);

		tv.tv_sec = 1;
		tv.tv_usec = 0;
		tmpfd= select( maxfd+ 1, &readfds, NULL, NULL, &tv);
		if( tmpfd <= 0)
			continue;

		tt= time( NULL);
		if( FD_ISSET( portfd1, &readfds))
		{
			 len= SerialBlockRead( port1, buf, 1024);
			 buf[ len]= '\0';
			 printf("port%d - %s\n", port1+ 1, buf);
			 SerialWrite( port2, buf, len);

			if( logfd1 != -1)
			{
				write( logfd1, ctime( &tt), strlen( ctime( &tt)));
				write( logfd1, buf, len);
				write( logfd1, "\n", 1);
				LCMPrint( ctime( &tt));
				LCMPrint( buf);
			}
		}
		if( FD_ISSET( portfd2, &readfds))
		{
			len= SerialBlockRead( port2, buf, 1024);
			buf[ len]= '\0';
			printf("port%d - %s\n", port2+ 1, buf);
			SerialWrite( port1, buf, len);
			 
			if( logfd2 != -1)
			{
				write( logfd2, ctime( &tt), strlen( ctime( &tt)));
				write( logfd2, buf, len);
				write( logfd2, "\n", 1);
				LCMPrint( ctime( &tt));
				LCMPrint( buf);
			}
		}
	}
	
	return 0;
}

